#!/bin/bash

# sh2html.sh <src_file> -- colorize a shell script for html display

# Convert a shell script into HTML, with syntax highlighting 
# 
# (C) 2004 Kevin Waldron 
# Released under the terms of the GNU General Public Licence 
# 
# Most modern Bourne Shell features are implemented here. Other 
# common external commands are also implemented. We can't implement 
# everything that may exist in everyones PATH's however!! 

# Modified to perform coverage display
# robert.brown.contractor@8451.com
# Thu, Feb 04, 2016  3:47:43 PM


# validate command line
if [ "$#" -ne "1" ]; then 
   echo "Usage: $(basename $0) filename" && exit 1 
fi 

# define input, output, and coverage files
I_FILE=$1
O_FILE=${1}.html
C_FILE=${1}.cov

if [ ! -e "${I_FILE}" ]; then 
   echo "$(basename $0): Error: ${I_FILE} does not exist" && exit 1 
fi 

if [ ! -e "${C_FILE}" ]; then 
   echo "$(basename $0): Error: ${C_FILE} does not exist" && exit 1 
fi 

# main awk routine - syntax highlighting done here! 
awk '{ 

    # these gsubs globally replace special chars 
    gsub( "&", "\\&amp;", $0 ); 
    gsub( "<", "\\&lt;", $0 ); 
    gsub( ">", "\\&gt;", $0 ); 

 
          commandArray[0] = "awk"; 
	  commandArray[1] = "read"; 
	  commandArray[2] = "exit"; 
	  commandArray[3] = "ls"; 
	  commandArray[4] = "rm"; 
	  commandArray[5] = "sort"; 
	  commandArray[6] = "uniq"; 
	  commandArray[7] = "sed"; 
	  commandArray[8] = "mv"; 
	  commandArray[9] = "cp"; 
	  commandArray[10] = "join"; 
	  commandArray[11] = "grep"; 
	  commandArray[12] = "cat"; 
	  commandArray[13] = "df"; 
	  commandArray[14] = "cd"; 
	  commandArray[15] = "basename"; 
	  commandArray[16] = "id"; 
	  commandArray[17] = "tr"; 
	  commandArray[18] = "ls"; 
	  commandArray[19] = "true"; 
	  commandArray[20] = "false"; 
	  commandArray[21] = "seq"; 
	  commandArray[22] = "cut"; 
	  commandArray[23] = "getopt"; 
	  commandArray[24] = "getopts"; 
	  commandArray[25] = "gunzip"; 
	  commandArray[26] = "gzip"; 
	  commandArray[27] = "tar"; 
	  commandArray[28] = "date"; 
	  commandArray[29] = "clear"; 
	  commandArray[30] = "ps"; 
	  commandArray[31] = "ifconfig"; 
	  commandArray[32] = "ping"; 
	  commandArray[33] = "tput"; 
	  commandArray[34] = "useradd"; 
	  commandArray[35] = "passwd"; 
	  commandArray[36] = "stty"; 
	  commandArray[37] = "chown"; 
	  commandArray[38] = "chmod"; 
	  commandArray[39] = "trap"; 
	  commandArray[40] = "sleep"; 
	  commandArray[41] = "wait"; 
	  commandArray[42] = "last"; 
	  commandArray[43] = "head"; 
	  commandArray[44] = "let"; 
	  commandArray[45] = "shift"; 
	  commandArray[46] = "unalias"; 
	  commandArray[47] = "read"; 
	  commandArray[48] = "unset"; 
	  commandArray[49] = "alias"; 
	  commandArray[50] = "expr"; 
	  commandArray[51] = "printf"; 
	  commandArray[52] = "mkdir"; 
	  commandArray[53] = "declare"; 
	  commandArray[54] = "local"; 
	  commandArray[55] = "tail"; 
          commandArray[56] = "hostname"; 
	  commandArray[57] = "xterm"; 
	  commandArray[58] = "bc"; 
	  commandArray[59] = "wc"; 
	  commandArray[60] = "more"; 
	  commandArray[61] = "find"; 
	  commandArray[62] = "mount"; 
	  commandArray[63] = "who"; 
	  commandArray[64] = "suspend"; 
	  commandArray[65] = "ulimit"; 
	  commandArray[66] = "umask"; 
	  commandArray[67] = "source"; 
	  commandArray[68] = "bind"; 
	  commandArray[69] = "builtin"; 
	  commandArray[70] = "typeset"; 
	  commandArray[71] = "export"; 
	  commandArray[72] = "fc"; 
	  commandArray[73] = "history"; 
	  commandArray[74] = "hash"; 
	  commandArray[75] = "jobs"; 
	  commandArray[76] = "kill"; 
	  commandArray[77] = "fg"; 
	  commandArray[78] = "bg"; 
	  commandArray[79] = "logout"; 
	  commandArray[80] = "popd"; 
	  commandArray[81] = "pushd"; 
	  commandArray[82] = "pwd"; 
	  commandArray[83] = "shopt"; 
	  commandArray[84] = "test"; 
	  commandArray[85] = "touch"; 
 
      # update array_length manually below! 
	  array_length = 85; 
 
    # deal with comments first - ignore entire line if one is found 
    if ( $0 ~ /^[ 	]*#/ ) { 
         print "<font color=\"green\"><i>"$0"</i></font>"; 
    } 
    else { 
       # syntax highlight on a line-by-line, element-by-element basis 
       # dealing with echo firstly, so we can say echo "echo if else" 
       # and only have the "echo" highlighted 
       if ( $0 ~ /^[ 	]*echo/ ) { 
          gsub( /^[ 	]*echo/, "<b>&</b>", $0 ); 
          gsub( /;[ 	]*echo/, "; <b>echo</b>", $0 ); 
 
          # check for commands within ` and ( in echo 
          for ( i = 0; i <= array_length; i++ ) { 
	     x=commandArray[i]; 
 
	     if ( $0 ~ x ) { 
	         reg = "[(]"x"[ 	]*"; 
		 gsub( reg, "(<b>"x"</b> ", $0 ); 
		 reg = "[(][ ]+"x"[ 	]+"; 
		 gsub( reg, "( <b>"x"</b> ", $0 ); 
		 reg = "[`]"x"[ 	]*"; 
		 gsub( reg, "`<b>"x"</b> ", $0 ); 
		 reg = "[`][ ]+"x"[ 	]+"; 
		 gsub( reg, "` <b>"x"</b> ", $0 ); 
		 reg = "^[ 	]*"x"[ 	]*$"; 
		 gsub( reg, "<b>&</b>", $0 ); 
		 reg = "[ ]+"x"[ 	]+"; 
		 gsub( reg, "<b>&</b>", $0 ); 
	     } 
	  } 
       } 
       else { 
          # catch other echos - i.e. with case 
          gsub( /[ 	]*echo[ 	]+/, "<b>&</b>", $0 ); 
 
          # if - will be at start of line (usually) 
          gsub( /if test/, 
                "<font color=\"blue\"><b>if</b></font> <b>test</b>", $0 ); 
	      gsub( /^[ 	]*if[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
	      gsub( /^[ 	]*if[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # else - will be at start of line (usually) 
          gsub( /^[ 	]*else[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # elif - will be at start of line (usually) 
          gsub( /^[ 	]*elif[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # fi - will also usually be at the start of a line 
          gsub( /^[ 	]*fi[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # for - will be at start of line (usually) 
          gsub( /^[ 	]*for[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # select - will be at start of line (usually) 
          gsub( /^[ 	]*select[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # while - will be at start of line (usually) 
          gsub( /while read/, 
	        "<font color=\"blue\"><b>while</b></font> <b>read</b>", $0 ); 
	      gsub( /^[ 	]*while[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # until - will be at start of line (usually) 
          gsub( /^[ 	]*until[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # case - will be at start of line (usually) 
          gsub( /^[ 	]*case[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # esac - will be at start of line (usually) 
          gsub( /^[ 	]*esac[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # function - will be at start of line (usually) 
          gsub( /^[ 	]*function[ 	]+/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # done - will be at start of line (usually) 
          gsub( /^[ 	]*done[ 	]*/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
 
          # then - can be at start of line, or after a ; 
          gsub( /^[ 	]*then[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
          gsub( /;[ 	]*then[ 	]*$/, 
                "; <font color=\"blue\"><b>then</b></font>", $0 ); 
 
          # special case - then followed by comment 
          if ( /[ 	]*then[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /then/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
          # special case - do followed by comment 
          if ( /[ 	]*do[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /do/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
	      # special case - else followed by comment 
          if ( /[ 	]*else[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /else/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
	      # special case - break followed by comment 
          if ( /[ 	]*break[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /break/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
	      # special case - continue followed by comment 
          if ( /[ 	]*continue[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /continue/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
	      # special case - done followed by comment 
          if ( /[ 	]*done[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /done/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
	      # special case - fi followed by comment 
          if ( /[ 	]*fi[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /fi/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
	      # special case - esac followed by comment 
          if ( /[ 	]*esac[ 	]*#/ ) { 
               gsub( /[^$]#.*$/, 
                   "<font color=\"green\"><i>&</i></font>", $0 ); 
               gsub( /esac/, 
                   "<font color=\"blue\"><b>&</b></font>", $0 ); 
          } 
 
          # do - can be at start of line, or after a ; 
          gsub( /^[ 	]*do[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
          gsub( /;[ 	]*do[ 	]*$/, 
                "; <font color=\"blue\"><b>do</b></font>", $0 ); 
 
	      # continue - can be at start of line, or after a ; 
          gsub( /^[ 	]*continue[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
          gsub( /;[ 	]*continue[ 	]*$/, 
                "; <font color=\"blue\"><b>continue</b></font>", $0 ); 
	      gsub( /continue;/, 
                "<font color=\"blue\"><b>continue</b></font>;", $0 ); 
 
	      # break - can be at start of line, or after a ; or ending in ; 
          gsub( /^[ 	]*break[ 	]*$/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
          gsub( /;[ 	]*break[ 	]*$/, 
                "; <font color=\"blue\"><b>break</b></font>", $0 ); 
	      gsub( /break[ ]*;/, 
                "<font color=\"blue\"><b>break</b></font> ;", $0 ); 
 
	      # return - can be at start of line, or after a ; 
          gsub( /^[ 	]*return[ 	]*/, 
                "<font color=\"blue\"><b>&</b></font>", $0 ); 
          gsub( /;[ 	]*return[ 	]*/, 
                "; <font color=\"blue\"><b>return</b></font>", $0 ); 
 
	      # scan for more comments - probably makes some of the code 
          # above redundant! A bit of a "catch all" for comments that 
          # slip through the net. 
          if ( /[^${#]#[^#]/ ) { 
             gsub( /[^$^]#.*$/, 
                 "<font color=\"green\"><i>&</i></font>", $0 ); 
          } 
 
	      # exec - special case as more commands may follow it 
	      if ( /[^#][	]*exec[		]*/ ) { 
	         gsub( /exec/, "<b>&</b>", $0 ); 
	      } 
          if ( /^exec/ ) { gsub ( /exec/, "<b>exec</b>", $0 ); } 
 
 
	      # eval - special case as more commands may follow it 
	      if ( /[^#][	]*eval[		]*/ ) { 
	         gsub( /eval/, "<b>&</b>", $0 ); 
	      } 
 
	      # set - to allow tput reset 
	      if ( /set/ ) { 
	         gsub( /^[	]*set/, "<b>&</b>", $0 ); 
	      } 
 
	  for ( i = 0; i <= array_length; i++ ) { 
	     x=commandArray[i]; 
 
	     if ( $0 ~ x ) { 
	         reg = "[(]"x"[ 	]*"; 
		 gsub( reg, "(<b>"x"</b> ", $0 ); 
		 reg = "[(][ ]+"x"[ 	]+"; 
		 gsub( reg, "( <b>"x"</b> ", $0 ); 
		 reg = "[`]"x"[ 	]*"; 
		 gsub( reg, "`<b>"x"</b> ", $0 ); 
		 reg = "[\047]"x"[ 	]*"; 
		 gsub( reg, "\047<b>"x"</b> ", $0 ); 
		 reg = "[\042]"x"[ 	]*"; 
		 gsub( reg, "\042<b>"x"</b> ", $0 ); 
		 reg = "[`][ ]+"x"[ 	]+"; 
		 gsub( reg, "` <b>"x"</b> ", $0 ); 
		 reg = "[ 	]+"x"[ 	]+"; 
		 gsub( reg, "<b>&</b>", $0 ); 
		 reg = x";"; 
		 gsub( reg, "<b>"x"</b>;", $0 ); 
		 reg = "^[ 	]*"x"[ 	]+"; 
		 gsub( reg, "<b>&</b>", $0 ); 
		 reg = "[ ]*"x"[`]"; 
		 gsub( reg, " <b>"x"</b>`", $0 ); 
		 reg = "[ ]*"x"[)]"; 
		 gsub( reg, " <b>"x"</b>)", $0 ); 
		 reg = "[ ]*"x"[ ]+[`]"; 
		 gsub( reg, " <b>"x"</b> `", $0 ); 
		 reg = "[ ]*"x"[ ]+[)]"; 
		 gsub( reg, " <b>"x"</b> )", $0 ); 
		 reg = "^[ 	]*"x"[ 	]*$"; 
		 gsub( reg, "<b>&</b>", $0 ); 
		 reg = "[|][ ]+"x; 
		 gsub( reg, "| <b>"x"</b>", $0 ); 
	     } 
	  } 
 
          # some comparison ops 
          gsub( /[ 	]+\-gt[ 	]+/, " <b>-gt</b> ", $0 ); 
          gsub( /[ 	]+\-lt[ 	]+/, " <b>-lt</b> ", $0 ); 
          gsub( /[ 	]+\-eq[ 	]+/, " <b>-eq</b> ", $0 ); 
          gsub( /[ 	]+\-ne[ 	]+/, " <b>-ne</b> ", $0 ); 
          gsub( /[ 	]+\-ge[ 	]+/, " <b>-ge</b> ", $0 ); 
          gsub( /[ 	]+\-le[ 	]+/, " <b>-le</b> ", $0 ); 
          gsub( /[ 	]+\-a[ 	]+/, " <b>-a</b> ", $0 ); 
          gsub( /[ 	]+\-o[ 	]+/, " <b>-o</b> ", $0 ); 
	      gsub( /[ 	]+\-n[ 	]+/, " <b>-n</b> ", $0 ); 
          gsub( /[ 	]+![ 	]+/, "<b> !</b> ", $0 ); 
          gsub( /[ 	]+\-e[ 	]+/, " <b>-e</b> ", $0 ); 
          gsub( /[ 	]+\-z[ 	]+/, " <b>-z</b> ", $0 ); 
          gsub( /[ 	]+\-f[ 	]+/, " <b>-f</b> ", $0 ); 
       } 
       # print out the line now we are all gsubbed 
       print $0 
    } 
}' ${I_FILE} > ${O_FILE}.1
# formated shell code is in ${O_FILE}.1

# add coverage data
./merge_cov.sh ${O_FILE}.1 $C_FILE > $O_FILE.2

# Print standard header stuff to file 
( 
    cat <<EOF
<html>
<head>
<!-- Generated with sh2html - http://www.zazzybob.com -->
<title>$(basename ${I_FILE})</title>
</head>
<body>
EOF

    # show path/filename
    path_file=`awk '{ print $1 ; exit }' ${C_FILE}`
    file=`basename $path_file`
    path=`echo $path_file $file | awk '{ print substr($0,1,index($0,$2)-1) }' -`

    run_date=`date`
    cat <<EOF
<table border=0 bgcolor="#b0b0b0" cellpadding=5 cellspacing=0  width=100% align=center>
<tr><td align=left><larger><b>$run_date</b></td><td align=right><larger><b>$path<font color="white">$file</b></td></tr>
</table>
EOF


    # colorize each line according to its coverage
    awk -v dq='"' '
                   # start of source file display
                   BEGIN                                                  { print "<table border=0 bgcolor=" dq "#ffffff" dq " cellpadding=0 cellspacing=0  width=80% align=center>" }

                   # blank line
		   NF == 2                                                { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; next }

                   # entire line is a comment
                   $4 ~ "color=" dq "green" dq "><i>"                     { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; comments++ ; next }

                   # "done" keyword does not get executed
                   $4 ~ "color=" dq "blue" dq "><b>done</b>"            ||
                   $4 == "color=" dq "blue" dq "><b>" && $5 ~ "done</b>"  { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # "do" keyword does not get executed
                   $4 ~ "color=" dq "blue" dq "><b>do</b></font>"       ||
                   $4 == "color=" dq "blue" dq "><b>" && $5 ~ "do</b>"    { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # "if" keyword does not get executed
                   $4 ~ "color=" dq "blue" dq "><b>if</b></font>"       ||
                   $4 == "color=" dq "blue" dq "><b>" && $5 ~ "if</b>"    { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # "then" keyword does not get executed
                   $4 ~ "color=" dq "blue" dq "><b>then</b></font>"     ||
                   $4 == "color=" dq "blue" dq "><b>" && $5 ~ "then</b>"  { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # "else" keyword does not get executed
                   $4 ~ "color=" dq "blue" dq "><b>else</b></font>"     ||
                   $4 == "color=" dq "blue" dq "><b>" && $5 ~ "else</b>"  { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # "fi" keyword does not get executed
                   $4 ~ "color=" dq "blue" dq "><b>fi</b></font>"       ||
                   $4 == "color=" dq "blue" dq "><b>" && $5 ~ "fi</b>"    { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # curley brace on a line by itself does not get executed
                   $3 == "{"                                              { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }
                   $3 == "}"                                              { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # POSIX style function definition does not get executed
                   substr($3,length($3)-1) == "()"                        { print "<tr><td align=right width=5%> &nbsp </td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; key_word++ ; next }

                   # executable line that did not get executed
		   $2 == "0" && first_one != 0                            { print "<tr bgcolor=" dq "#ffdddd" dq "><td align=right width=5%>" $2 "</td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; not_covered++ ; next }
		   $2 == "0" && first_one == 0                            { first_one = 1 ; $2 = 1 }

                   # DEFAULT: executable line that did get executed
			                                                  { print "<tr bgcolor=" dq "#ddffdd" dq "><td align=right width=5%>" $2 "</td><td align=right width=5%>" $1 "</td><td width=2%></td><td align=left><pre><tt>" substr($0,length($1 " " $2 " ")+1) "</tt></pre></td></tr>" ; covered++ ; next }

                   # end of source file display
		   END                                                    { print "</table>" ; printf("%d %d %d %d\n", comments, covered, not_covered, key_word) }' ${O_FILE}.2 > ${O_FILE}.3

    # coverage statistics

    total_lines=`cat $path_file | wc -l`

    stats=`tail -n 1 ${O_FILE}.3`
    comments=`echo $stats | awk '{ print $1 }' -`
    covered=`echo $stats | awk '{ print $2 }' -`
    not_covered=`echo $stats | awk '{ print $3 }' -`
    key_word=`echo $stats | awk '{ print $4 }' -`

    executable=$(( $covered + $not_covered ))
    blank=$(( $total_lines - $executable - $comments - $key_word))

    comments_pct=$(( $(( 100 * $comments )) / $total_lines ))
    executable_pct=$(( $(( 100 * $executable )) / $total_lines ))
    covered_pct=$(( $(( 100 * $covered )) / $executable ))

    cat <<EOF
<table border=0 bgcolor="#d0d0d0" cellpadding=5 cellspacing=0  width=100% align=center>
<tr><td align=left width=5%></td><td align=left>  Coverage:    </td><td align=right>  $covered_pct     </td><td align=left width=15%>  %  </td><td align=left>  Covered:      </td><td align=right>  $covered      </td><td width=15%>&nbsp</td><td align=left>   Comment Lines:  </td><td align=right>  $comments     </td><td align=left width=5%></td></tr>
<tr><td align=left width=5%></td><td align=left>  Executable:  </td><td align=right>  $executable_pct  </td><td align=left width=15%>  %  </td><td align=left>  Not Covered:  </td><td align=right>  $not_covered  </td><td width=15%>&nbsp</td><td align=left>   Blank Lines:    </td><td align=right>  $blank        </td><td align=left width=5%></td></tr>
<tr><td align=left width=5%></td><td align=left>  Comments:    </td><td align=right>  $comments_pct    </td><td align=left width=15%>  %  </td><td align=left>  Executable :  </td><td align=right>  $executable   </td><td width=15%>&nbsp</td><td align=left>   Total Lines:    </td><td align=right>  $total_lines  </td><td align=left width=5%></td></tr>
</table>
EOF


    echo '<p />'
    echo '&nbsp'
    echo '<p />'

    cat <<EOF
<pre>
EOF

    # output the html listing
    head -n -1 ${O_FILE}.3

    # print standard footer stuff to file 
    cat <<EOF
</pre>
</body>
</html>
EOF
) > ${O_FILE}

# clean-up temp files
rm -f ${O_FILE}.1
rm -f ${O_FILE}.2
rm -f ${O_FILE}.3

exit 0 
