#!/bin/bash

# tr2cov.sh -- convert trace output file to coverage/profiling format

# reform multi-line entries into a singleline
awk 'substr($0,1,1) == "/" { print "" }
                           { printf("%s", $0) }' trace.log > tr2cov.0

# Make a file with 1 line per source line, and the count of how many
# times it was executed.
# <path/file> <line_num> <exec_cnt>
cat tr2cov.0 \
 | tr ':' ' ' \
 | cut -d ' ' -s -f1,2 \
 | sort \
 | uniq -c \
 | awk '{ print $2,$3,$1 }' - > tr2cov.1

# Make a list of all the source files that were traced
files=`awk '{ print $1 }' tr2cov.1 \
        | sort -u`

# Make a file for each source file, with a line for each line that was
# executed, showing line number and how many times it was executed.
# <path/file> <line_num> <exec_cnt>
for f in $files
do awk -v f=$f '$1 == f { print $0 }' tr2cov.1 \
    | sort -k2 -n > ${f}.cov
done

# get rid of temp files
rm -f tr2cov.0
rm -f tr2cov.1

